h1 {
    text-align: center;
}

.form-field {
    margin-left: auto;
    margin-right: auto;
    padding: 1em;
}

.field-name {
    display: inline-block;
    width: 45%;
    text-align: right;
    padding-right: 1em;
}

#submit-div {
    text-align: center;
}
