#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['nbparameterise', 'nbparameterise.code_drivers']

package_data = \
{'': ['*']}

setup(name='nbparameterise',
      version='0.5',
      description='Re-run a notebook substituting input parameters in the first cell.',
      author=None,
      author_email='Thomas Kluyver <thomas@kluyver.me.uk>',
      url=None,
      packages=packages,
      package_data=package_data,
      python_requires='>=3.6',
     )
